#!perl

use Test2::V0;
use Test::use::ok;

use ok 'Protocol::Sys::Virt';
use ok 'Protocol::Sys::Virt::URI';
use ok 'Protocol::Sys::Virt::Remote';
use ok 'Protocol::Sys::Virt::Transport';
use ok 'Protocol::Sys::Virt::KeepAlive';



done_testing;
